//-----------------------------------------------------------------------------
// Project            	GeometryKit
// Class		G3DBox
// Creator            	Philippe C.D. Robert
// Maintainer         	Philippe C.D. Robert
// Creation Date      	Thu Sep  9 11:05:08 CEST 1999
//
// Copyright (c) Philippe C.D. Robert
//
// The GeometryKit is free software; you can redistribute it and/or modify it 
// under the terms of the GNU LGPL Version 2 as published by the Free 
// Software Foundation
//
// $Id: G3DBox.h,v 1.8 2001/01/05 20:50:47 robert Exp $
//-----------------------------------------------------------------------------

#ifndef __G3DBox_h_INCLUDE
#define __G3DBox_h_INCLUDE

#import <Foundation/Foundation.h>
#import <GeometryKit/G3DIntersecting.h>

@class G3DMatrix4f;
@class G3DTuple3f;
@class G3DSphere;
@class G3DLine;
@class G3DPlane;

@interface G3DBox : NSObject <NSCoding, NSCopying, G3DIntersecting>
{
  G3DTuple3f *min;
  G3DTuple3f *max;
}

//-----------------------------------------------------------------------------
// class methods
//-----------------------------------------------------------------------------

+ (G3DBox *)box;
+ (G3DBox *)boxWithMin:(G3DTuple3f *)minimum max:(G3DTuple3f *)maximum;
+ (G3DBox *)boxWithBox:(G3DBox *)aBox;

//-----------------------------------------------------------------------------
// init and free
//-----------------------------------------------------------------------------

- (id)init;
- (id)initWithMin:(G3DTuple3f *)minimum max:(G3DTuple3f *)maximum;
- (id)initWithBox:(G3DBox *)aBox;

- (void)dealloc;

//-----------------------------------------------------------------------------
// math
//-----------------------------------------------------------------------------

- (void)wrapPoint:(G3DTuple3f *)pt;
- (void)wrapBox:(G3DBox *)b;
- (void)wrapSphere:(G3DSphere *)s;

- (BOOL)intersectsObject:(id)anObject;

- (BOOL)intersectsLine:(G3DLine *)l;
- (BOOL)intersectsBox:(G3DBox *)aBox;
  // currently it tests whether aBox is contained within self

- (BOOL)intersectsPlane:(G3DPlane *)aPlane;
- (BOOL)intersectsSphere:(G3DSphere *)aSphere;
  // Invokes the appropriate method in G3DSphere

- (BOOL)wrapsPoint:(G3DTuple3f *)pt;

- (void)transform:(G3DMatrix4f *)m;
  // Calculates m * min and m * max, such as it can be used by OpenGL. 

//-----------------------------------------------------------------------------
// accessor methods
//-----------------------------------------------------------------------------

- (void)setMin:(G3DTuple3f *)minVec max:(G3DTuple3f *)maxVec;
- (void)getMin:(G3DTuple3f **)minVec max:(G3DTuple3f **)maxVec;

- (void)setMin:(G3DTuple3f *)m;
- (G3DTuple3f *)min;

- (void)setMax:(G3DTuple3f *)m;
- (G3DTuple3f *)max;

- (float)volume;
- (BOOL)isEmpty;

- (BOOL)isEqualToBox:(G3DBox *)aBox;

//-----------------------------------------------------------------------------
// NSCoding
//-----------------------------------------------------------------------------

- (void)encodeWithCoder:(NSCoder *)aCoder;
- (id)initWithCoder:(NSCoder *)aCoder;

//-----------------------------------------------------------------------------
// NSCopying
//-----------------------------------------------------------------------------

- (id)copyWithZone:(NSZone *)zone;

@end

extern NSString *G3DBoxException;

#endif




