//-----------------------------------------------------------------------------
// Project            	RenderKit
// Class		G3DMatrix3d
// Creator            	Philippe C.D. Robert
// Maintainer         	Philippe C.D. Robert
// Creation Date      	Thu Sep  9 11:05:08 CEST 1999
//
// Copyright (c) Philippe C.D. Robert
//
// The GeometryKit is free software; you can redistribute it and/or modify it 
// under the terms of the GNU LGPL Version 2 as published by the Free 
// Software Foundation
//
// $Id: G3DMatrix3d.h,v 1.9 2001/01/05 20:50:48 robert Exp $
//-----------------------------------------------------------------------------

#ifndef __G3DMatrix3d_h_INCLUDE
#define __G3DMatrix3d_h_INCLUDE

#import <Foundation/Foundation.h>
#import <GeometryKit/G3DDefs.h>

@class G3DTuple3d;
@class G3DVector3d;
@class G3DQuaternion;

@interface G3DMatrix3d : NSObject <NSCoding, NSCopying>
{
  @public

  double matrix[9];     // the matrix values in row order

  @protected
  
  int _mode;           // G3D_MatrixType

  double _t[9];        // the transposed
  double _i[9];        // the inverted
}

//-----------------------------------------------------------------------------
// class methods
//-----------------------------------------------------------------------------

+ (void)initialize;

+ (G3DMatrix3d *)matrix3d;
+ (G3DMatrix3d *)matrix3dWithMatrix3d:(G3DMatrix3d *)aMatrix;
+ (G3DMatrix3d *)matrix3dWithElements:(double *)vals;

//-----------------------------------------------------------------------------
// init and free
//-----------------------------------------------------------------------------

- (id)init;
- (id)initWithMatrix3d:(G3DMatrix3d *)aMatrix;
- (id)initWithElements:(double *)vals;

- (void)dealloc;

//-----------------------------------------------------------------------------
// math
//-----------------------------------------------------------------------------

- (void)addMatrix:(G3DMatrix3d *)a;
- (void)subMatrix:(G3DMatrix3d *)a;

- (void)multiplyByMatrix:(G3DMatrix3d *)a;
- (void)multiplyByMatrix:(G3DMatrix3d *)a and:(G3DMatrix3d *)b;
- (void)multiplyByMatrix:(G3DMatrix3d *)a andTransposed:(G3DMatrix3d *)b;
- (void)multiplyByTransposedMatrix:(G3DMatrix3d *)a and:(G3DMatrix3d *)b;
- (void)multiplyByTransposedMatrix:(G3DMatrix3d *)a andTransposed:(G3DMatrix3d *)b;

- (void)multiplyByMatrices:(NSArray *)matrices;

- (G3DTuple3d *)vectorByPreMultiplying:(G3DTuple3d *)aTuple;
// Returns aTuple * self;
- (G3DTuple3d *)vectorByPostMultiplying:(G3DTuple3d *)aTuple;
// Returns self * aTuple

- (BOOL)invert;
- (G3DMatrix3d *)invertedMatrix;

- (void)transpose;
- (G3DMatrix3d *)transposedMatrix;

- (void)addScalar:(double)scalar;
- (void)subScalar:(double)scalar;

- (void)multiplyByScalar:(double)scalar;
- (void)divideByScalar:(double)scalar;

- (void)multiplyRow:(int)pos byValue:(double)val;
- (void)multiplyCol:(int)pos byValue:(double)val;

- (void)addElements:(double *)vals toRow:(int)rowPos;
- (void)addElements:(double *)vals toColumn:(int)colPos;

- (void)subElements:(double *)vals fromRow:(int)rowPos;
- (void)subElements:(double *)vals fromColumn:(int)colPos;

- (double)determinant;

//-----------------------------------------------------------------------------
// accessor methods
//-----------------------------------------------------------------------------

- (void)setMatrix:(G3DMatrix3d *)aMatrix;

- (void)setXRotation:(double)angle;
- (void)setYRotation:(double)angle;
- (void)setZRotation:(double)angle;
- (void)setRotation:(double)angle axis:(double *)axis;
- (void)setRotation:(double)angle axisVector:(G3DVector3d *)vec;

- (G3DVector3d *)scale;
- (void)setScale:(G3DVector3d *)factor;

- (void)setElements:(double *)vals;
- (double *)elements;

- (G3DVector3d *)rowVectorAtIndex:(int)row;
- (G3DVector3d *)columnVectorAtIndex:(int)col;

- (void)setElement:(double)val atRow:(int)row column:(int)col;
- (double)elementAtRow:(int)row column:(int)col;

//-----------------------------------------------------------------------------
// misc
//-----------------------------------------------------------------------------

- (NSString *)description;

- (BOOL)isEqualToMatrix:(G3DMatrix3d *)aMatrix;

//-----------------------------------------------------------------------------
// NSCoding
//-----------------------------------------------------------------------------

- (void)encodeWithCoder:(NSCoder *)aCoder;
- (id)initWithCoder:(NSCoder *)aCoder;

//-----------------------------------------------------------------------------
// NSCopying
//-----------------------------------------------------------------------------

- (id)copyWithZone:(NSZone *)zone;

@end

extern NSString *G3DMatrix3dException;

#endif




