//-----------------------------------------------------------------------------
// Project            	RenderKit
// Class		G3DMatrix3f
// Creator            	Philippe C.D. Robert
// Maintainer         	Philippe C.D. Robert
// Creation Date      	Thu Sep  9 11:05:08 CEST 1999
//
// Copyright (c) Philippe C.D. Robert
//
// The GeometryKit is free software; you can redistribute it and/or modify it 
// under the terms of the GNU LGPL Version 2 as published by the Free 
// Software Foundation
//
// $Id: G3DMatrix3f.h,v 1.8 2001/01/05 20:50:48 robert Exp $
//-----------------------------------------------------------------------------

#ifndef __G3DMatrix3f_h_INCLUDE
#define __G3DMatrix3f_h_INCLUDE

#import <Foundation/Foundation.h>
#import <GeometryKit/G3DDefs.h>

@class G3DTuple3f;
@class G3DVector3f;
@class G3DQuaternion;

@interface G3DMatrix3f : NSObject <NSCoding, NSCopying>
{
  @public

  float matrix[9];     // the matrix values in row order

  @protected
    
  int _mode;           // G3D_MatrixType

  float _t[9];         // the transposed
  float _i[9];         // the inverted
}

//-----------------------------------------------------------------------------
// class methods
//-----------------------------------------------------------------------------

+ (void)initialize;

+ (G3DMatrix3f *)matrix3f;
+ (G3DMatrix3f *)matrix3fWithMatrix3f:(G3DMatrix3f *)aMatrix;
+ (G3DMatrix3f *)matrix3fWithElements:(float *)vals;

//-----------------------------------------------------------------------------
// init and free
//-----------------------------------------------------------------------------

- (id)init;
- (id)initWithMatrix3f:(G3DMatrix3f *)aMatrix;
- (id)initWithElements:(float *)vals;

- (void)dealloc;

//-----------------------------------------------------------------------------
// math
//-----------------------------------------------------------------------------

- (void)addMatrix:(G3DMatrix3f *)a;
- (void)subMatrix:(G3DMatrix3f *)a;

- (void)multiplyByMatrix:(G3DMatrix3f *)a;
- (void)multiplyByMatrix:(G3DMatrix3f *)a and:(G3DMatrix3f *)b;
- (void)multiplyByMatrix:(G3DMatrix3f *)a andTransposed:(G3DMatrix3f *)b;
- (void)multiplyByTransposedMatrix:(G3DMatrix3f *)a and:(G3DMatrix3f *)b;
- (void)multiplyByTransposedMatrix:(G3DMatrix3f *)a andTransposed:(G3DMatrix3f *)b;

- (void)multiplyByMatrices:(NSArray *)matrices;

- (G3DTuple3f *)vectorByPreMultiplying:(G3DTuple3f *)aTuple;
// Returns aTuple * self;
- (G3DTuple3f *)vectorByPostMultiplying:(G3DTuple3f *)aTuple;
// Returns self * aTuple

- (BOOL)invert;
- (G3DMatrix3f *)invertedMatrix;

- (void)transpose;
- (G3DMatrix3f *)transposedMatrix;

- (void)addScalar:(float)scalar;
- (void)subScalar:(float)scalar;

- (void)multiplyByScalar:(float)scalar;
- (void)divideByScalar:(float)scalar;

- (void)multiplyRow:(int)pos byValue:(float)val;
- (void)multiplyCol:(int)pos byValue:(float)val;

- (void)addElements:(float *)vals toRow:(int)rowPos;
- (void)addElements:(float *)vals toColumn:(int)colPos;

- (void)subElements:(float *)vals fromRow:(int)rowPos;
- (void)subElements:(float *)vals fromColumn:(int)colPos;

- (float)determinant;

//-----------------------------------------------------------------------------
// accessor methods
//-----------------------------------------------------------------------------

- (void)setMatrix:(G3DMatrix3f *)aMatrix;

- (void)setXRotation:(float)angle;
- (void)setYRotation:(float)angle;
- (void)setZRotation:(float)angle;
- (void)setRotation:(float)angle axis:(float *)axis;
- (void)setRotation:(float)angle axisVector:(G3DVector3f *)vec;

- (G3DVector3f *)scale;
- (void)setScale:(G3DVector3f *)factor;

- (void)setElements:(float *)vals;
- (float *)elements;

- (G3DVector3f *)rowVectorAtIndex:(int)row;
- (G3DVector3f *)columnVectorAtIndex:(int)col;

- (void)setElement:(float)val atRow:(int)row column:(int)col;
- (float)elementAtRow:(int)row column:(int)col;

//-----------------------------------------------------------------------------
// misc
//-----------------------------------------------------------------------------

- (NSString *)description;

- (BOOL)isEqualToMatrix:(G3DMatrix3f *)aMatrix;

//-----------------------------------------------------------------------------
// NSCoding
//-----------------------------------------------------------------------------

- (void)encodeWithCoder:(NSCoder *)aCoder;
- (id)initWithCoder:(NSCoder *)aCoder;

//-----------------------------------------------------------------------------
// NSCopying
//-----------------------------------------------------------------------------

- (id)copyWithZone:(NSZone *)zone;

@end

extern NSString *G3DMatrix3fException;

#endif




