//-----------------------------------------------------------------------------
// Project            	GeometryKit
// Class		G3DTuple4d
// Creator            	Philippe C.D. Robert
// Maintainer         	Philippe C.D. Robert
// Creation Date      	Thu Sep  9 11:05:08 CEST 1999
//
// Copyright (c) Philippe C.D. Robert
//
// The GeometryKit is free software; you can redistribute it and/or modify it 
// under the terms of the GNU LGPL Version 2 as published by the Free 
// Software Foundation
//
// $Id: G3DTuple4d.h,v 1.5 2001/01/05 20:50:49 robert Exp $
//-----------------------------------------------------------------------------

#ifndef __G3DTuple4d_h_INCLUDE
#define __G3DTuple4d_h_INCLUDE

#import <Foundation/Foundation.h>

@interface G3DTuple4d : NSObject <NSCoding, NSCopying>
{
  double tuple[4];
}

//-----------------------------------------------------------------------------
// Class methods
//-----------------------------------------------------------------------------

+ (void)initialize;

+ (G3DTuple4d *)tuple4d;
+ (G3DTuple4d *)tuple4dWithElements:(double *)vals;
+ (G3DTuple4d *)tuple4dWithX:(double)x y:(double)y z:(double)z w:(double)w;
+ (G3DTuple4d *)tuple4dWithTuple4d:(G3DTuple4d *)aTuple;

//-----------------------------------------------------------------------------
// init and free
//-----------------------------------------------------------------------------

- (id)init;
// Initialises a new tuple with the values 0.0,0.0,0.0,0.0

- (id)initWithElements:(double *)vals;
- (id)initWithX:(double)x y:(double)y z:(double)z w:(double)w;
  // Designated initialiser

- (id)initWithTuple:(G3DTuple4d *)aTuple;
  // Initialises a new tuple by invoking initWithElements

//-----------------------------------------------------------------------------
// Math
//-----------------------------------------------------------------------------

- (void)absolute;

- (void)clamp;
- (void)clampLow:(double)low high:(double)high;

- (void)addTuple4d:(G3DTuple4d *)aTuple;
- (void)subTuple4d:(G3DTuple4d *)aTuple;

- (void)multiplyBy:(double)aScalar;
- (void)divideBy:(double)aScalar;

- (void)interpolateBetween:(G3DTuple4d *)first and:(G3DTuple4d *)second factor:(double)factor;
// (1-factor)*first + factor*second

- (void)negate;

- (BOOL)isEqualToTuple:(id)anObj;

//-----------------------------------------------------------------------------
// Accessing
//-----------------------------------------------------------------------------

- (double *)elements;
- (void)setElements:(double *)values;
- (void)getElements:(double *)values;

- (double)x;
- (void)setX:(double)x;

- (double)y;
- (void)setY:(double)y;

- (double)z;
- (void)setZ:(double)z;

- (double)w;
- (void)setW:(double)w;

- (void)setValuesWithTuple:(G3DTuple4d *)aTuple;

- (NSString *)description;

//-----------------------------------------------------------------------------
// NSCoding
//-----------------------------------------------------------------------------

- (void)encodeWithCoder:(NSCoder *)aCoder;
- (id)initWithCoder:(NSCoder *)aCoder;

//-----------------------------------------------------------------------------
// NSCopying
//-----------------------------------------------------------------------------

- (id)copyWithZone:(NSZone *)zone;

@end

extern NSString *G3DTuple4dException;

#endif




