//-----------------------------------------------------------------------------
// Project            	GeometryKit
// Class		G3DVector
// Creator            	Philippe C.D. Robert
// Maintainer         	Philippe C.D. Robert
// Creation Date      	Thu Sep  9 11:05:08 CEST 1999
//
// Copyright (c) Philippe C.D. Robert
//
// The GeometryKit is free software; you can redistribute it and/or modify it 
// under the terms of the GNU LGPL Version 2 as published by the Free 
// Software Foundation
//
// $Id: G3DVector.h,v 1.6 2001/01/05 20:50:49 robert Exp $
//-----------------------------------------------------------------------------

#ifndef __G3DVector_h_INCLUDE
#define __G3DVector_h_INCLUDE

#import <Foundation/Foundation.h>

@class G3DMatrix;

@interface G3DVector : NSObject <NSCoding, NSCopying>
{
  double *vector;

  @protected

  int _size;
}

//-----------------------------------------------------------------------------
// Class methods
//-----------------------------------------------------------------------------

+ (G3DVector *)vectorWithSize:(int)val;
+ (G3DVector *)vectorWithValues:(double *)vals size:(int)val;
+ (G3DVector *)vectorWithVector:(G3DVector *)vec;

//-----------------------------------------------------------------------------
// init and free
//-----------------------------------------------------------------------------

- (id)initWithSize:(int)val;
  // Initialises a new vector by invoking initWithElements

- (id)initWithElements:(double *)vals size:(int)val;
// Designated initialiser

- (id)initWithVector:(G3DVector *)vec;
  // Initialises a new vector by invoking initWithElements

- (void)dealloc;

//-----------------------------------------------------------------------------
// Math
//-----------------------------------------------------------------------------

- (void)vectorByAdding:(G3DVector *)vec;
- (void)vectorBySubtracting:(G3DVector *)vec;

- (void)vectorByMultiplyingBy:(double)aScalar;
- (void)vectorByDividingBy:(double)aScalar;

- (void)interpolateBetween:(G3DVector *)first and:(G3DVector *)second factor:(double)factor;
// (1-factor)*first + factor*second

- (void)vectorByMultiplyingMatrix:(G3DMatrix *)m and:(G3DVector *)vec;
// self = m * vec

- (void)vectorByMultiplyingVector:(G3DVector *)vec and:(G3DMatrix *)m;
// self = transpose(vec) * m

- (double)dotProduct:(G3DVector *)aVec;

- (double)norm;

- (double)squaredLength;

- (void)negate;

- (void)zero;

- (void)normalise;

- (G3DVector *)normalisedVector;

- (BOOL)isEqualToVector:(G3DVector *)vec;

//-----------------------------------------------------------------------------
// Accessing
//-----------------------------------------------------------------------------

- (int)size;

- (double *)elements;
- (void)setElements:(double *)el;

- (void)setElement:(double)el atIndex:(int)index;
- (double)elementAtIndex:(int)index;

- (NSString *)description;

//-----------------------------------------------------------------------------
// NSCoding
//-----------------------------------------------------------------------------

- (void)encodeWithCoder:(NSCoder *)aCoder;
- (id)initWithCoder:(NSCoder *)aCoder;

//-----------------------------------------------------------------------------
// NSCopying
//-----------------------------------------------------------------------------

- (id)copyWithZone:(NSZone *)zone;

@end

extern NSString *G3DVectorException;

#endif







