//-----------------------------------------------------------------------------
// Project            	RenderKit
// Class		G3DArchiver
// Creator            	Philippe C.D. Robert
// Maintainer         	Philippe C.D. Robert
// Creation Date      	Wed Nov 24 18:49:25 CET 1999
//
// Copyright (c) Philippe C.D. Robert
//
// The RenderKit is free software; you can redistribute it and/or modify it 
// under the terms of the GNU LGPL Version 2 as published by the Free 
// Software Foundation
//
// $Id: G3DArchiver.h,v 1.12 2001/07/07 15:35:47 probert Exp $
//-----------------------------------------------------------------------------

/*
 Description: 

 A lot of this class and the general archiving concept is borrowed from
 Ovidiu Predescu's GMArchiving. Basically it mimics the OpenStep archiving
 concept in a readable ASCII (plist) format.

 */

#ifndef __G3DArchiver_h_INCLUDE
#define __G3DArchiver_h_INCLUDE

#import <Foundation/Foundation.h>

#import <RenderKit/G3DSceneCoding.h>
#import <RenderKit/G3DTypes.h>
#import <RenderKit/G3DFrustum.h>

@interface G3DArchiver : NSObject
{
  NSMutableDictionary *pList;
  NSMutableArray *topLevelObjects;
  id lastObjectRep;
  NSMapTable *objects;         // object -> name
  NSHashTable *conditionals;   // conditional objects
  NSMapTable *classes;         // real classname -> class info

  int           counter;
  int           level;

  BOOL          writingRoot;   // YES if encodeRootObject:withName: was sent
  BOOL findingConditionals;    // YES if finding conditionals
}

//-----------------------------------------------------------------------------
// Class methods
//-----------------------------------------------------------------------------

+ (BOOL)archiveObjectTree:(id)obj toFile:(NSString *)path;
  // Invokes archivedDataWithTreeRoot and writeToFile

+ (id)archivedObjectTreeWithRoot:(id)obj;
  // Archives an object tree into a readable ASCII propertylist

//-----------------------------------------------------------------------------
// Init and free
//-----------------------------------------------------------------------------

- (id)init;
- (void)dealloc;

//-----------------------------------------------------------------------------
// Archiving
//-----------------------------------------------------------------------------

- (NSDictionary *)propertyList;

- (NSString*)newLabel;

- (BOOL)writeToFile:(NSString*)path;

//-----------------------------------------------------------------------------
// Encoding Objects
//-----------------------------------------------------------------------------

- (id)encodeRootObject:(id)rootObject named:(NSString*)name;

- (id)encodeConditionalObject:(id)object named:(NSString*)name;
- (id)encodeObject:(id)anObject named:(NSString*)name;

- (id)encodeString:(NSString*)anObject named:(NSString*)name;
- (id)encodeArray:(NSArray*)array named:(NSString*)name;
- (id)encodeDictionary:(NSDictionary*)dictionary named:(NSString*)name;
- (id)encodeData:(NSData*)anObject named:(NSString*)name;

- (id)encodeClass:(Class)class named:(NSString*)name;
- (id)encodeSelector:(SEL)selector named:(NSString*)name;

//-----------------------------------------------------------------------------
// Encoding C Types
//-----------------------------------------------------------------------------

- (void)encodeBytes:(const void *)b length:(unsigned)l named:(NSString *)name;

- (void)encodeArrayOfTypeDoubleAt:(const double *)address count:(unsigned)cnt named:(NSString*)name;

- (void)encodeArrayOfTypeFloatAt:(const float *)address count:(unsigned)cnt named:(NSString*)name;

- (void)encodeArrayOfTypeIntAt:(const int *)address count:(unsigned)cnt named:(NSString*)name;

- (void)encodeChar:(char)value named:(NSString*)name;
- (void)encodeUnsignedChar:(unsigned char)value named:(NSString*)name;

- (void)encodeBOOL:(BOOL)value named:(NSString*)name;

- (void)encodeShort:(short)value named:(NSString*)name;
- (void)encodeUnsignedShort:(unsigned short)value named:(NSString*)name;

- (void)encodeInt:(int)value named:(NSString*)name;
- (void)encodeUnsignedInt:(unsigned int)value named:(NSString*)name;

- (void)encodeLong:(long)value named:(NSString*)name;
- (void)encodeUnsignedLong:(unsigned long)value named:(NSString*)name;

- (void)encodeFloat:(float)value named:(NSString*)name;
- (void)encodeDouble:(double)value named:(NSString*)name;

//-----------------------------------------------------------------------------
// Encoding other DTs
//-----------------------------------------------------------------------------


- (void)encodePoint:(NSPoint)point named:(NSString*)name;
- (void)encodeSize:(NSSize)size named:(NSString*)name;
- (void)encodeRect:(NSRect)rect named:(NSString*)name;

- (void)encodeFrustum:(G3DFrustumValues)fr named:(NSString*)name;
- (void)encodeCoordinate:(G3DCoordinate)coord named:(NSString*)name;
- (void)encodeG3DRange:(G3DRange)range named:(NSString*)name;
- (void)encodeBoundingSphere:(G3DBoundingSphere)bsph named:(NSString*)name;
- (void)encodeBoundingBox:(G3DBoundingBox)bbox named:(NSString*)name;

@end

@interface NSObject (G3DSceneArchiving)

+ (id)createObjectForSceneUnarchiver:(G3DUnarchiver *)unarchiver;

- (id)replacementObjectForSceneArchiver:(G3DArchiver *)archiver;
- (Class)classForSceneArchiver;

@end

extern NSString *G3DArchiverRootObjectDuplicationException;
extern NSString *G3DArchiverObjectNotWrittenException;

#endif





