//-----------------------------------------------------------------------------
// Project            	RenderKit
// Class		G3DBVol
// Creator            	Philippe C.D. Robert
// Maintainer         	Philippe C.D. Robert
// Creation Date      	Sun Jun 25 10:35:50 CEST 2000
//
// Copyright (c) Philippe C.D. Robert
//
// The RenderKit is free software; you can redistribute it and/or modify it 
// under the terms of the GNU LGPL Version 2 as published by the Free 
// Software Foundation
//
// $Id: G3DBVol.h,v 1.14 2001/07/07 15:35:47 probert Exp $
//-----------------------------------------------------------------------------

#ifndef __G3DBVol_h_INCLUDE
#define __G3DBVol_h_INCLUDE

#import <Foundation/Foundation.h>
#import <RenderKit/G3DAttributeNode.h>
#import <RenderKit/G3DGeometryBackend.h>
#import <GeometryKit/G3DIntersecting.h>

@class G3DTransform;
@class G3DGraphNode;
@class G3DMatrix4f;

typedef enum {
  G3D_Sphere = 0,
  G3D_AABB,
  G3D_OBB
} G3D_BVolType;

@interface G3DBVol : G3DAttributeNode <NSCopying,NSCoding,G3DSceneCoding>
{
  id<NSObject,G3DIntersecting> volumeObject;
  id<NSObject,G3DIntersecting> transformedVolumeObject;
  
  int volumeType; // G3D_BVolType
  float volume;

  G3DGraphNode *node;

  @private
    
  G3DMatrix4f *_tmpMat;
  BOOL _vObjIsDirty;
  BOOL _trvoNeedsUpdate;
}

//-----------------------------------------------------------------------------
// Init and free
//-----------------------------------------------------------------------------

- (id)initWithType:(G3D_BVolType)volType;
  // Designated initialiser. In order to use the volume, first invoke updateWithGeometry or mergeWithBVolume!

- (id)initWithType:(G3D_BVolType)volType geometry:(id<G3DGeometryBackend>)geom;
  // Invokes initWithType and updateWithGeometry

//-----------------------------------------------------------------------------
// Updating
//-----------------------------------------------------------------------------

- (void)mergeWithBVolume:(G3DBVol *)bVol;
  // Merging of volumes of different types results in the volume with the smallest possible volume.

- (void)updateWithGeometry:(id<G3DGeometryBackend>)geom;

//-----------------------------------------------------------------------------
// Accessor methods
//-----------------------------------------------------------------------------

- (G3D_BVolType)type;

- (void)setNode:(G3DGraphNode *)node;
- (G3DGraphNode *)node;

- (id<G3DIntersecting>)volumeObject;
- (id<G3DIntersecting>)transformedVolumeObject;

//-----------------------------------------------------------------------------
// Enclosing
//-----------------------------------------------------------------------------

- (float)volume;

- (BOOL)intersectsWithVolume:(G3DBVol *)vol;

//-----------------------------------------------------------------------------
// NSCoding
//-----------------------------------------------------------------------------

- (void)encodeWithCoder:(NSCoder *)aCoder;
- (id)initWithCoder:(NSCoder *)aCoder;

//-----------------------------------------------------------------------------
// NSCopying
//-----------------------------------------------------------------------------

- (id)copyWithZone:(NSZone *)zone;

//-----------------------------------------------------------------------------
// G3DSceneCoding
//-----------------------------------------------------------------------------

- (void)encodeWithSceneArchiver:(G3DArchiver *)archiver;
- (id)initWithSceneUnarchiver:(G3DUnarchiver *)unarchiver;

@end

@interface G3DBVol (RenderingExtensions)
   
- (void)updateVisObj;  
- (void)renderBoundingVolume;

@end

extern NSString *G3DBVolTypeException;
extern NSString *G3DBVolException;

#endif




