//-----------------------------------------------------------------------------
// Project            	RenderKit
// Class		G3DCamera
// Creator            	Philippe C.D. Robert
// Maintainer         	Philippe C.D. Robert
// Creation Date      	Thu Sep  9 11:05:08 CEST 1999
//
// Copyright (c) Philippe C.D. Robert
//
// The RenderKit is free software; you can redistribute it and/or modify it 
// under the terms of the GNU LGPL Version 2 as published by the Free 
// Software Foundation
//
// $Id: G3DCamera.h,v 1.10 2001/07/15 13:55:43 probert Exp $
//-----------------------------------------------------------------------------

#ifndef __G3DCamera_h_INCLUDE
#define __G3DCamera_h_INCLUDE

#import <Foundation/Foundation.h>
#import <RenderKit/G3DTypes.h>

@class G3DFrustum;
@class G3DWorld;
@class G3DColour;
@class G3DTransform;
@class G3DQuaternion;
@class G3DState;
@class G3DFog;

@interface G3DCamera : NSObject 
{
  G3DWorld     *world; 
  G3DFrustum   *frustum;
  G3DState     *state;
  G3DFog       *fog;
  G3DColour    *backgroundColour; 
  G3DTransform *preTransformation; 

  G3D_CameraPosition position; 
  G3D_CameraFlags    cameraFlags; 

  id renderDelegate;
  NSSize _size;

  @private

  BOOL     _hasDelegate;
  BOOL     _notifyBefore;
  BOOL     _notifyAfter;
  BOOL     _useTwoSideModel;
  float    _rdist[4];
  unsigned _rMask;
}

//-----------------------------------------------------------------------------
// Context handling
//-----------------------------------------------------------------------------

+ (G3DCamera *)activeCamera;
+ (void)setActiveCamera:(G3DCamera *)aCamera;

+ (void)measureFramerates:(BOOL)yn;

//-----------------------------------------------------------------------------
// Init
//-----------------------------------------------------------------------------

- (id)initWithSize:(NSSize)aSize;

//-----------------------------------------------------------------------------
// Delegate
//-----------------------------------------------------------------------------

- (id)renderDelegate;
- (void)setRenderDelegate:(id)aDelegate;

//-----------------------------------------------------------------------------
// Light information
//-----------------------------------------------------------------------------

- (void)setEnableLighting:(BOOL)flag;
- (BOOL)enableLighting;

//-----------------------------------------------------------------------------
// Rendering information
//-----------------------------------------------------------------------------

- (void)setCullingEnabled:(BOOL)yn;
- (BOOL)cullingEnabled;

- (void)setUseTwoSideModel:(BOOL)flag;
- (BOOL)useTwoSideModel;

- (G3DFrustum *)frustum;
- (G3DState *)state;

//-----------------------------------------------------------------------------
// Background drawing 
//-----------------------------------------------------------------------------

- (void)setFog:(G3DFog *)f;
- (G3DFog *)fog;

- (void)setBackgroundColour:(G3DColour *)bg;
- (G3DColour *)backgroundColour;
- (BOOL)doesDrawBackgroundColour;
- (void)setDrawBackgroundColour:(BOOL)yn;

//-----------------------------------------------------------------------------
// Accessor methods
//-----------------------------------------------------------------------------

- (void)setWorld:(G3DWorld *)aWorld;
- (G3DWorld *)world;

- (void)setViewportWithSize:(NSSize)s;

- (NSSize)size;
- (G3D_CameraPosition *)position;

//-----------------------------------------------------------------------------
// Drawing
//-----------------------------------------------------------------------------

- (void)setupCamera;

- (void)preprocessScene;

- (void)render;

//-----------------------------------------------------------------------------
// Transformation
//-----------------------------------------------------------------------------

- (void)setPreTransformation:(G3DTransform *)tf;
- (G3DTransform *)preTransformation;

- (void)setUsePreTransformation:(BOOL)yn;
- (BOOL)usePreTransformation;

//-----------------------------------------------------------------------------
// Camera Movements
//-----------------------------------------------------------------------------

- (void)moveX:(const float)x;
- (void)moveY:(const float)y;
- (void)moveZ:(const float)z;
- (void)moveX:(const float)x y:(const float)y z:(const float)z;

- (void)rotateX:(const float)angle;
- (void)rotateY:(const float)angle;
- (void)rotateZ:(const float)angle;
- (void)rotateX:(const float)xA y:(const float)yA z:(const float)zA;

- (void)rotateByAngleAxis:(float *)vec4;
- (void)rotateByQuaternion:(G3DQuaternion *)quat;

- (void)setCamera:(const float *)pos toward:(const float *)to up:(const float *)up;

@end

@interface NSObject (G3DCameraDelegate)

- (void)cameraDidInitialise:(id)sender;

- (void)cameraDidPreprocessScene:(id)sender;
- (void)cameraWillPreprocessScene:(id)sender;

- (void)cameraViewportDidChange:(id)sender;

- (void)cameraWillRenderFrame:(id)sender;
- (void)cameraDidRenderFrame:(id)sender;

@end

extern NSString *G3DCameraSetupException;

#endif



