//-----------------------------------------------------------------------------
// Project            	RenderKit
// Class		G3DFog
// Creator            	Philippe C.D. Robert
// Maintainer         	Philippe C.D. Robert
// Creation Date      	Thu Sep  9 11:05:08 CEST 1999
//
// Copyright (c) Philippe C.D. Robert
//
// The RenderKit is free software; you can redistribute it and/or modify it 
// under the terms of the GNU LGPL Version 2 as published by the Free 
// Software Foundation
//
// $Id: G3DFog.h,v 1.6 2001/01/05 20:50:51 robert Exp $
//-----------------------------------------------------------------------------

/*
 * Fog is enabled while preprocessing the world to render. Soif you want to
 * make customisations to the fog, you have to do it before preprocessing it!
 * After that, any changes you make are not updated, unless you call
 * G3DBindFog() again.
 *
 */

#ifndef __G3DFog_h_INCLUDE
#define __G3DFog_h_INCLUDE

#import <Foundation/Foundation.h>

#import <RenderKit/G3DTypes.h>
#import <RenderKit/G3DColour.h>
#import <RenderKit/G3DRenderEngine.h>

typedef enum {
  G3D_LINEAR_FOG = GL_LINEAR,
  G3D_EXP_FOG = GL_EXP
} G3D_FogMode;

@interface G3DFog : NSObject
{
  G3DColour *colour;

  double density;
  double end;
  double start;
  
  G3D_FogMode mode;
}

//-----------------------------------------------------------------------------
// Class method
//-----------------------------------------------------------------------------

+ (G3DFog *)fog;
+ (G3DFog *)fogWithColour:(G3DColour *)col density:(double)dens start:(double)st end:(double)en fogMode:(G3D_FogMode)mode;

//-----------------------------------------------------------------------------
// Init and free
//-----------------------------------------------------------------------------

- (id)init;
// Initialises a white fog in linear mode, start = 0, end = 100 and density = 0.5

- (void)dealloc;

//-----------------------------------------------------------------------------
// Accessor methods
//-----------------------------------------------------------------------------

- (void)setFogColour:(G3DColour *)col;
- (G3DColour *)fogColour;

- (double)end;
- (void)setEnd:(double)val;

- (double)start;
- (void)setStart:(double)val;

- (double)density;
- (void)setDensity:(double)val;

- (G3D_FogMode)fogMode;
- (void)setFogMode:(G3D_FogMode)val;
- (void)cycleFogMode;

- (NSString *)description;

@end

#endif




