//-----------------------------------------------------------------------------
// Project            	RenderKit
// Class		G3DFrustum
// Creator            	Philippe C.D. Robert
// Maintainer         	Philippe C.D. Robert
// Creation Date      	Thu Sep  9 11:05:08 CEST 1999
//
// Copyright (c) Philippe C.D. Robert
//
// The RenderKit is free software; you can redistribute it and/or modify it 
// under the terms of the GNU LGPL Version 2 as published by the Free 
// Software Foundation
//
// $Id: G3DFrustum.h,v 1.12 2001/01/05 20:50:51 robert Exp $
//-----------------------------------------------------------------------------

#ifndef __G3DFrustum_h_INCLUDE
#define __G3DFrustum_h_INCLUDE

#import <Foundation/Foundation.h>
#import <GeometryKit/GeometryKit.h>

#import <RenderKit/G3DDefines.h>
#import <RenderKit/G3DTypes.h>

@class G3DBVol;

typedef struct _G3DFrustum {
  float left;
  float right;
  float top;
  float bottom;
  float near;
  float far;
  float hfov;
  float vfov;
} G3DFrustumValues;

typedef enum {
  G3DPerspective,
  G3DOrthographic
} G3DProjectionType;

@interface G3DFrustum : NSObject
{
  G3DFrustumValues frustum;

  float top[4];     // Top plaen
  float bottom[4];  // Bottom plane
  float right[4];   // Right plane
  float left[4];    // Left plane

  @private

  G3DProjectionType projectionType;

  float _tr_left[4];
  float _tr_right[4];
  float _tr_top[4];
  float _tr_bottom[4];

  float _currentPos[3];
  float _maxRad;
}

//-----------------------------------------------------------------------------
// init and free
//-----------------------------------------------------------------------------

- (id)init;
- (id)initWithLeft:(float)l right:(float)r top:(float)t bottom:(float)b near:(float)n far:(float)f;

//-----------------------------------------------------------------------------
// Frustum handling
//-----------------------------------------------------------------------------

- (void)update;

- (void)transformWithRotation:(float *)mat position:(float *)pos;

//-----------------------------------------------------------------------------
// Accessor methods
//-----------------------------------------------------------------------------

- (void)setHFOV:(float)hAngle vFOV:(float)vAngle;
- (void)setNear:(float)n far:(float)f;

- (void)setFrustumValues:(G3DFrustumValues)aFrustum;
- (G3DFrustumValues)frustumValues;

- (void)setProjectionType:(G3DProjectionType)tp;
- (G3DProjectionType)projectionType;

- (G3DMatrix4f *)projectionMatrix;

//-----------------------------------------------------------------------------
// Testing
//-----------------------------------------------------------------------------

- (G3DLocation)containsVolume:(G3DBVol *)vol;

- (G3DLocation)containsSphere:(G3DSphere *)sph;
- (G3DLocation)containsAABB:(G3DBox *)box;
- (G3DLocation)containsOBB:(G3DBox *)box;

- (BOOL)containsVector:(G3DVector4d *)aVec;

@end

extern NSString *G3DFrustumException;

#endif



