//-----------------------------------------------------------------------------
// Project            	RenderKit
// Class		G3DLODGroup
// Creator            	Philippe C.D. Robert
// Maintainer         	Philippe C.D. Robert
// Creation Date      	Thu Sep  9 11:05:08 CEST 1999
//
// Copyright (c) Philippe C.D. Robert
//
// The RenderKit is free software; you can redistribute it and/or modify it 
// under the terms of the GNU LGPL Version 2 as published by the Free 
// Software Foundation
//
// $Id: G3DLODGroup.h,v 1.9 2001/01/05 20:50:52 robert Exp $
//-----------------------------------------------------------------------------

#ifndef __G3DLODGroup_h_INCLUDE
#define __G3DLODGroup_h_INCLUDE

#import <Foundation/Foundation.h>
#import <RenderKit/G3DSwitchGroup.h>

/*
 * G3D_LODMode defines the way the indices are calculated. G3D_Linear is based
 * on equal distances each child "occupies", G3D_Exp weightens the near
 * distances more than far distances, which may lead to better rendering
 * results.
 */

typedef enum {
  G3D_Linear,
  G3D_Exp
} G3D_LODMode;

@interface G3DLODGroup : G3DSwitchGroup <NSCoding, NSCopying, G3DSceneCoding>
{
  float maxDistance;
  int mode; // G3D_LODMode
}

//-----------------------------------------------------------------------------
// Init and free
//-----------------------------------------------------------------------------

- (id)initWithCapacity:(unsigned int)cap maxDistance:(float)val;

//-----------------------------------------------------------------------------
// Level of detail
//-----------------------------------------------------------------------------

- (void)setMaxDistance:(float)val;
- (float)maxDistance;

- (void)setLodMode:(G3D_LODMode)md;
- (G3D_LODMode)lodMode;

//-----------------------------------------------------------------------------
// NSCoding
//-----------------------------------------------------------------------------

- (void)encodeWithCoder:(NSCoder *)aCoder;
- (id)initWithCoder:(NSCoder *)aCoder;

//-----------------------------------------------------------------------------
// NSCopying
//-----------------------------------------------------------------------------

- (id)copyWithZone:(NSZone *)zone;

//-----------------------------------------------------------------------------
// G3DSceneCoding
//-----------------------------------------------------------------------------

- (void)encodeWithSceneArchiver:(G3DArchiver *)archiver;
- (id)initWithSceneUnarchiver:(G3DUnarchiver *)unarchiver;

@end

extern NSString *G3DLODGroupException;

#endif




