//-----------------------------------------------------------------------------
// Project            	RenderKit
// Class		G3DMaterial
// Creator            	Philippe C.D. Robert
// Maintainer         	Philippe C.D. Robert
// Creation Date      	Thu Sep  9 11:05:08 CEST 1999
//
// Copyright (c) Philippe C.D. Robert
//
// The RenderKit is free software; you can redistribute it and/or modify it 
// under the terms of the GNU LGPL Version 2 as published by the Free 
// Software Foundation
//
// $Id: G3DMaterial.h,v 1.10 2001/01/05 20:50:52 robert Exp $
//-----------------------------------------------------------------------------

#ifndef __G3DMaterial_h_INCLUDE
#define __G3DMaterial_h_INCLUDE

#import <Foundation/Foundation.h>

#import <RenderKit/G3DTypes.h>
#import <GeometryKit/GeometryKit.h>
#import <RenderKit/G3DAttributeNode.h>

typedef enum {
  G3D_Ambient = GL_AMBIENT,
  G3D_Diffuse = GL_DIFFUSE,
  G3D_AmbientDiffuse = GL_AMBIENT_AND_DIFFUSE,
  G3D_Specular = GL_SPECULAR,
  G3D_Emission = GL_EMISSION
} G3D_MaterialMode;

typedef enum {
  G3D_Front = GL_FRONT,
  G3D_Back = GL_BACK,
  G3D_FrontBack = GL_FRONT_AND_BACK
} G3D_MaterialFace;

@class G3DColour;
@class G3DState;

@interface G3DMaterial : G3DAttributeNode <NSCoding, NSCopying, G3DSceneCoding>
{
  G3DColour *ambient;
  G3DColour *emissive;
  G3DColour *diffuse;
  G3DColour *specular;

  double shininess; 	// 0..128

  int materialMode;     // G3D_MaterialMode
  int face;             // G3D_MaterialFace

  G3DState *state;	// Not retained!
}

//-----------------------------------------------------------------------------
// Class methods
//-----------------------------------------------------------------------------

+ (G3DMaterial *)materialWithState:(G3DState *)st;

//-----------------------------------------------------------------------------
// Init and free
//-----------------------------------------------------------------------------

- (id)initWithState:(G3DState *)aState;
- (void)dealloc;

//-----------------------------------------------------------------------------
// Accessor methods
//-----------------------------------------------------------------------------

- (void)setState:(G3DState *)aState;
- (G3DState *)state;

- (void)setAmbientColour:(G3DColour *)aCol;
- (G3DColour *)ambientColour;

- (void)setEmissiveColour:(G3DColour *)eCol;
- (G3DColour *)emissiveColour;

- (void)setDiffuseColour:(G3DColour *)dCol;
- (G3DColour *)diffuseColour;

- (void)setSpecularColour:(G3DColour *)sCol;
- (G3DColour *)specularColour;

- (void)setShininess:(double)sh;
- (double)shininess;

- (void)setMaterialMode:(G3D_MaterialMode)mode;
- (G3D_MaterialMode)materialMode;

- (void)setFace:(G3D_MaterialFace)fc;
- (G3D_MaterialFace)face;
// Front, back or both - this needs some redesign, since we could have 2 separate settings for it!

- (NSString *)description;

//-----------------------------------------------------------------------------
// NSCoding
//-----------------------------------------------------------------------------

- (void)encodeWithCoder:(NSCoder *)aCoder;
- (id)initWithCoder:(NSCoder *)aCoder;

//-----------------------------------------------------------------------------
// NSCopying
//-----------------------------------------------------------------------------

- (id)copyWithZone:(NSZone *)zone;

//-----------------------------------------------------------------------------
// G3DSceneCoding
//-----------------------------------------------------------------------------

- (void)encodeWithSceneArchiver:(G3DArchiver *)archiver;
- (id)initWithSceneUnarchiver:(G3DUnarchiver *)unarchiver;

@end

#endif




