//-----------------------------------------------------------------------------
// Project            	RenderKit
// Class		G3DNodeList
// Creator            	Philippe C.D. Robert
// Maintainer         	Philippe C.D. Robert
// Creation Date      	Tue May  2 15:34:06 CEST 2000
//
// Copyright (c) Philippe C.D. Robert
//
// The RenderKit is free software; you can redistribute it and/or modify it 
// under the terms of the GNU LGPL Version 2 as published by the Free 
// Software Foundation
//
// $Id: G3DNodeList.h,v 1.6 2001/07/15 12:39:24 probert Exp $
//-----------------------------------------------------------------------------

#ifndef __G3DNodeList_h_INCLUDE
#define __G3DNodeList_h_INCLUDE

#import <Foundation/Foundation.h>

@class G3DGraphNode;
@class G3DNodeEnumerator;
@class G3DFrustum;

@interface G3DNodeList : NSObject
{
  id *nodes;

  @private

  int _cnt;
  int _cap;
  int _curr_ind;
}

//-----------------------------------------------------------------------------
// Init and free
//-----------------------------------------------------------------------------

- (id)initWithNodes:(NSArray *)nds;
// Invokes initWithCapacity: and adds the objects from nds to the list

- (id)initWithCapacity:(unsigned int)cap;
// Designated initialiser

- (void)dealloc;

//-----------------------------------------------------------------------------
// List accessing
//-----------------------------------------------------------------------------

- (void)renderWithMask:(unsigned)mask frustum:(G3DFrustum *)fr;

- (id*)nodes;
- (int)count;

- (G3DGraphNode *)nextNode;
// Returns the next node, starting with node at index 0. After the last node has been returned, it will start with the node at index 0, again.

- (unsigned int)indexOfNextNode;

- (G3DNodeEnumerator *)nodeEnumerator;

//-----------------------------------------------------------------------------
// List handling
//-----------------------------------------------------------------------------

- (void)addNode:(G3DGraphNode *)aNode;
// Raises a G3DNodeListAccessException if the list is complete - resizing is not yet supported!

- (void)removeNode:(G3DGraphNode *)aNode;
// Invokes removeNodeAtIndex to remove the node

- (void)removeNodeAtIndex:(unsigned int)index;
// Raises a G3DNodeListAccessException if index is not valid!

- (G3DGraphNode *)nodeAtIndex:(unsigned int)index;
// Raises a G3DNodeListAccessException if index is not valid!

@end

extern NSString *G3DNodeListAccessException;

#endif




