//-----------------------------------------------------------------------------
// Project            	RenderKit
// Class		G3DOverlay
// Creator            	Philippe C.D. Robert
// Maintainer         	Philippe C.D. Robert
// Creation Date      	Wed Jun 28 12:38:33 CEST 2000
//
// Copyright (c) Philippe C.D. Robert
//
// The RenderKit is free software; you can redistribute it and/or modify it 
// under the terms of the GNU LGPL Version 2 as published by the Free 
// Software Foundation
//
// $Id: G3DOverlay.h,v 1.11 2001/01/05 20:50:53 robert Exp $
//-----------------------------------------------------------------------------

/*
 * Overlays are rendered at the very end of the rendering process. They are
 * used to display 2D text and images on top of all other graphics. Since they
 * are 2D only, overlays refer only to x and y translation (and scaling).
 * Rotation is not supported with overlays!
 *
 */

#ifndef __G3DOverlay_h_INCLUDE
#define __G3DOverlay_h_INCLUDE

#import <Foundation/Foundation.h>
#import <RenderKit/G3DLeaf.h>

@class G3DTexture;
@class G3DState;

@interface G3DOverlay : G3DLeaf <NSCopying, NSCoding,G3DSceneCoding>
{
  G3DState *state;
  G3DTexture *texture;

  BOOL enableBlending;

  @private
    
  float _blendCol[4];
  int _w;
  int _h;
}

//-----------------------------------------------------------------------------
// Class methods
//-----------------------------------------------------------------------------

+ (G3DOverlay *)overlayWithTexture:(G3DTexture *)tx position:(G3DTuple2f*)p;

//-----------------------------------------------------------------------------
// Init and Free
//-----------------------------------------------------------------------------

- (id)initWithTexture:(G3DTexture *)tx position:(G3DTuple2f*)p;

//-----------------------------------------------------------------------------
// Accessor methods
//-----------------------------------------------------------------------------

- (void)setTexture:(G3DTexture *)tex;
- (G3DTexture *)texture;

- (void)setEnableBlending:(BOOL)yn;
- (BOOL)enableBlending;

//-----------------------------------------------------------------------------
// NSCoding
//-----------------------------------------------------------------------------

- (void)encodeWithCoder:(NSCoder *)aCoder;
- (id)initWithCoder:(NSCoder *)aCoder;

//-----------------------------------------------------------------------------
// NSCopying
//-----------------------------------------------------------------------------

- (id)copyWithZone:(NSZone *)zone;

//-----------------------------------------------------------------------------
// G3DSceneCoding
//-----------------------------------------------------------------------------

- (void)encodeWithSceneArchiver:(G3DArchiver *)archiver;
- (id)initWithSceneUnarchiver:(G3DUnarchiver *)unarchiver;

@end

#endif




