//-----------------------------------------------------------------------------
// Project            	RenderKit
// Class		G3DRenderEngine
// Creator            	Philippe C.D. Robert
// Maintainer         	Philippe C.D. Robert
// Creation Date      	Tue May  2 12:22:38 CEST 2000
//
// Copyright (c) Philippe C.D. Robert
//
// The RenderKit is free software; you can redistribute it and/or modify it 
// under the terms of the GNU LGPL Version 2 as published by the Free 
// Software Foundation
//
// $Id: G3DRenderEngine.h,v 1.20 2001/07/07 15:35:47 probert Exp $
//-----------------------------------------------------------------------------

/*
 * Description:
 *
 */

#ifndef __G3DRenderEngine_h_INCLUDE
#define __G3DRenderEngine_h_INCLUDE

#import <Foundation/Foundation.h>
#import <RenderKit/G3DDefines.h>
#import <RenderKit/G3DTypes.h>

#ifdef GNUSTEP
#include <GL/gl.h>
#else
#include <OpenGL/gl.h>
#endif

#define VIEW_FRUSTUM_CULLING 0
#define ENABLE_GL_LIGHTS     1
#define ENABLE_AL_SOUNDS     2

#ifndef GET_IMP
#define GET_IMP(X) ((void*)[self instanceMethodForSelector:(X)])
#endif

#define G3D_PUSH_MATRIX glPushMatrix()
#define G3D_POP_MATRIX  glPopMatrix()

#define MAX_DYN_LIGHTS	8

@class G3DFog;
@class G3DLight;
@class G3DColour;

/*
 * Miscellaneous
 *
 */

__G3DEI__ void G3DBindFog(G3DFog *fog);

/*
 * Lighting
 *
 */

__G3DEI__ void G3DEnableLighting(void);
__G3DEI__ void G3DDisableLighting(void);
__G3DEI__ int G3DEnableLight(G3DLight *light);
__G3DEI__ int G3DUpdateLight(G3DLight *light);
__G3DEI__ void G3DDisableLight(G3DLight *light);
__G3DEI__ int G3DCreateLightHandle(void);

@interface G3DRenderEngine : NSObject
{
}

//-----------------------------------------------------------------------------
// Class methods
//-----------------------------------------------------------------------------

+ (G3DRenderEngine *)renderEngine;

//-----------------------------------------------------------------------------
// Information and initialisations
//-----------------------------------------------------------------------------

- (void)initEngine;
- (void)cleanup;

- (void)setRenderMode:(G3D_RenderMode)mode;
- (G3D_RenderMode)renderMode;
- (void)toggleRenderMode;

//-----------------------------------------------------------------------------
// Lighting
//-----------------------------------------------------------------------------

- (void)enableVectorNormalisation;
- (void)disableVectorNormalisation;

- (void)setGlobalAmbientColour:(G3DColour *)aCol;

- (void)setUseInfiniteViewpoint:(BOOL)yn;

- (void)enableTwoSidedLighting;
- (void)disableTwoSidedLighting;

//-----------------------------------------------------------------------------
// Description
//-----------------------------------------------------------------------------

- (NSString *)description;

@end

#endif




