//-----------------------------------------------------------------------------
// Project            	RenderKit
// Class		G3DSensor
// Creator            	Philippe C.D. Robert
// Maintainer         	Philippe C.D. Robert
// Creation Date      	Thu Sep  9 11:05:08 CEST 1999
//
// Copyright (c) Philippe C.D. Robert
//
// The RenderKit is free software; you can redistribute it and/or modify it 
// under the terms of the GNU LGPL Version 2 as published by the Free 
// Software Foundation
//
// $Id: G3DSensor.h,v 1.9 2001/01/05 20:50:54 robert Exp $
//-----------------------------------------------------------------------------

#ifndef __G3DSensor_h_INCLUDE
#define __G3DSensor_h_INCLUDE

#import <Foundation/Foundation.h>

#import <RenderKit/G3DLeaf.h>
#import <RenderKit/G3DTrigger.h>

@interface G3DSensor : G3DLeaf <NSCopying, NSCoding, G3DSceneCoding>
{
  id<G3DTrigger> trigger;

  id observer;
  SEL selector;

  BOOL state;
}

//-----------------------------------------------------------------------------
// Class methods
//-----------------------------------------------------------------------------

+ (G3DSensor *)sensorWithTrigger:(id<G3DTrigger>)aTrigger observer:(id)obs selector:(SEL)aSelector;

//-----------------------------------------------------------------------------
// Init and free
//-----------------------------------------------------------------------------

- (id)initWithTrigger:(id<G3DTrigger>)aTrigger observer:(id)obs selector:(SEL)aSelector;

//-----------------------------------------------------------------------------
// Triggering
//-----------------------------------------------------------------------------

- (void)triggerIfNeeded:(G3DFrustum *)fr;
  // Triggering of sensor nodes takes place before everything else is rendered.

//-----------------------------------------------------------------------------
// Accessor methods
//-----------------------------------------------------------------------------

- (void)setState:(BOOL)onoff;
- (BOOL)state;

- (id)observer;

//-----------------------------------------------------------------------------
// NSCoding
//-----------------------------------------------------------------------------

- (void)encodeWithCoder:(NSCoder *)aCoder;
- (id)initWithCoder:(NSCoder *)aCoder;

//-----------------------------------------------------------------------------
// NSCopying
//-----------------------------------------------------------------------------

- (id)copyWithZone:(NSZone *)zone;

//-----------------------------------------------------------------------------
// G3DSceneCoding
//-----------------------------------------------------------------------------

- (void)encodeWithSceneArchiver:(G3DArchiver *)archiver;
- (id)initWithSceneUnarchiver:(G3DUnarchiver *)unarchiver;

@end

@interface NSObject (SensorDelegates)

- (BOOL)sensorShouldTrigger:(id)sender;

@end

#endif




