//-----------------------------------------------------------------------------
// Project            	RenderKit
// Class		G3DShape
// Creator            	Philippe C.D. Robert
// Maintainer         	Philippe C.D. Robert
// Creation Date      	Thu Sep  9 11:05:08 CEST 1999
//
// Copyright (c) Philippe C.D. Robert
//
// The RenderKit is free software; you can redistribute it and/or modify it 
// under the terms of the GNU LGPL Version 2 as published by the Free 
// Software Foundation
//
// $Id: G3DShape.h,v 1.28 2001/01/09 15:30:21 robert Exp $
//-----------------------------------------------------------------------------

#ifndef __G3DShape_h_INCLUDE
#define __G3DShape_h_INCLUDE

#import <Foundation/Foundation.h>
#import <GeometryKit/G3DMatrix4d.h>

#import <RenderKit/G3DLeaf.h>
#import <RenderKit/G3DBVol.h>
#import <RenderKit/G3DGeometryBackend.h>
#import <RenderKit/G3DState.h>

@class G3DTexture;
@class G3DMaterial;
@class G3DState;

@interface G3DShape : G3DLeaf <NSCoding,NSCopying,G3DSceneCoding>
{
  G3DState *state;

  G3DTexture *texture;
  id<NSObject,G3DGeometryBackend> geometry;

  int surface;  // G3DSurfaceType
  int bVolType;	// G3D_BVolType, default is G3D_Sphere
  
  BOOL _dlistIsDirty;
  BOOL _loadsLazy;
  BOOL _isLoaded;
}

//-----------------------------------------------------------------------------
// Class methods
//-----------------------------------------------------------------------------

+ (G3DShape *)shape;
+ (G3DShape *)shapeWithTexture:(G3DTexture *)aTexture geometry:(id<NSObject,G3DGeometryBackend>)geom;

//-----------------------------------------------------------------------------
// Init and free
//-----------------------------------------------------------------------------

- (id)init;
- (id)initWithTexture:(G3DTexture *)aTexture geometry:(id<NSObject,G3DGeometryBackend>)geom;

//-----------------------------------------------------------------------------
// Accessor methods
//-----------------------------------------------------------------------------

- (void)setBVolType:(G3D_BVolType)tp;
  // Marks the bounding volume as dirty, if _hasBVolume

- (G3D_BVolType)bVolType;

- (void)setBoundingVolume:(G3DBVol *)bVolume;
  // Marks the parent's bounding volume as dirty and changes itself to _hasBVolume, if bVolume is not nil, otherwise _hasBVolume is set to NO

- (void)setGeometry:(id<NSObject,G3DGeometryBackend>)aGeom;
  // Copies aGeom and marks the bounding volume as dirty, if _hasBVolume. One can then set a bVolume manually w/o having it computed, if needed.

- (id<G3DGeometryBackend>)geometry;

- (void)setTexture:(G3DTexture *)tex;
- (G3DTexture *)texture;
  // Set the texture object, sets up the texture if needed and enables texturing

- (void)setState:(G3DState *)aState;
- (G3DState *)state;

//-----------------------------------------------------------------------------
// State facade methods
//-----------------------------------------------------------------------------

- (G3DMaterial *)material;

- (void)setEnableTexturing:(BOOL)flag;
- (BOOL)enableTexturing;

- (void)setEnableBlending:(BOOL)flag;
- (BOOL)enableBlending;

- (void)setEnableBackfaceCulling:(BOOL)flag;
- (BOOL)enableBackfaceCulling;

- (void)setShadeMode:(G3D_ShadeMode)mode;
- (G3D_ShadeMode)shadeMode;

- (void)setEnableLighting:(BOOL)flag;
- (BOOL)enableLighting;

//-----------------------------------------------------------------------------
// NSCopying
//-----------------------------------------------------------------------------

- (id)copyWithZone:(NSZone *)zone;

//-----------------------------------------------------------------------------
// Encoding/Decoding
//-----------------------------------------------------------------------------

- (void)encodeWithCoder:(NSCoder *)aCoder;
- (id)initWithCoder:(NSCoder *)aCoder;

//-----------------------------------------------------------------------------
// G3DSceneCoding
//-----------------------------------------------------------------------------

- (void)encodeWithSceneArchiver:(G3DArchiver *)archiver;
- (id)initWithSceneUnarchiver:(G3DUnarchiver *)unarchiver;

@end

#endif




