//-----------------------------------------------------------------------------
// Project            	RenderKit
// Class		G3DState
// Creator            	Philippe C.D. Robert
// Maintainer         	Philippe C.D. Robert
// Creation Date      	Thu Sep  9 11:05:08 CEST 1999
//
// Copyright (c) Philippe C.D. Robert
//
// The RenderKit is free software; you can redistribute it and/or modify it 
// under the terms of the GNU LGPL Version 2 as published by the Free 
// Software Foundation
//
// $Id: G3DState.h,v 1.14 2001/01/05 20:50:54 robert Exp $
//-----------------------------------------------------------------------------

#ifndef __G3DState_h_INCLUDE
#define __G3DState_h_INCLUDE

#import <Foundation/Foundation.h>

#import <RenderKit/G3DTypes.h>
#import <RenderKit/G3DAttributeNode.h>

#define G3D_TEXTURE_EN        0
#define G3D_CULL_FACE_EN      1
#define G3D_COLOR_MATERIAL_EN 2
#define G3D_BLEND_EN          3
#define G3D_ALPHA_TEST_EN     4
#define G3D_LIGHTING_EN       5
 
#define G3D_TEXTURE           6
#define G3D_COLOR_MATERIAL    7
#define G3D_DIFFUSE           8
#define G3D_AMBIENT           9
#define G3D_SPECULAR         10
#define G3D_EMISSION         11
#define G3D_SHININESS        12
#define G3D_ALPHA_TEST       13
#define G3D_SHADE_MODEL      14

typedef enum {
  G3D_Flat = GL_FLAT,
  G3D_Smooth = GL_SMOOTH
} G3D_ShadeMode;

extern void (*__G3DEnableTable[64]) ();
extern void (*__G3DDisableTable[64])();

@class G3DState;
extern void enableState(G3DState *self, int s_feature);

typedef enum {
  G3D_Never = GL_NEVER,
  G3D_Always = GL_ALWAYS,
  G3D_Less = GL_LESS,
  G3D_Equal = GL_EQUAL,
  G3D_LEqual = GL_LEQUAL,
  G3D_GEqual = GL_GEQUAL,
  G3D_Greater = GL_GREATER,
  G3D_NotEqual = GL_NOTEQUAL
} G3D_AlphaFunc;

typedef enum {
  G3D_Texturing = 0,
  G3D_Culling,
  G3D_ColourMaterial,
  G3D_Lighting,
  G3D_Blending,
  G3D_AlphaTest
} G3D_RenderFeature;

@class G3DMaterial;

@interface G3DState : G3DAttributeNode <NSCoding, NSCopying, G3DSceneCoding>
{
  G3DMaterial *material;   // the shape's material information
  unsigned textureHandle;   // tex handle if used, 0 otherwise

  BOOL isTranslucent;     // needed to optimise GL
  int shadeMode;          // G3D_ShadeMode, default is G3D_Smooth

  int alphaFunc;          // G3D_AlphaFunc, function for alpha testing
  float alphaClamp;

  int ignore;             // use this state or ignore it
  int enable;             // enable a specific feature or not
}

//-----------------------------------------------------------------------------
// Class methods
//-----------------------------------------------------------------------------

+ (G3DState *)state;
// Returns a simple, initial state

+ (G3DState *)currentState;
// Returns the state of the current rendering context

//-----------------------------------------------------------------------------
// init and free
//-----------------------------------------------------------------------------

- (id)init;
- (id)initWithMaterial:(G3DMaterial *)m;

//-----------------------------------------------------------------------------
// State methods
//-----------------------------------------------------------------------------

- (void)activate;
- (void)activateIfNeeded;

- (BOOL)isTranslucent;
- (void)setIsTranslucent:(BOOL)yn;

- (void)setShadeMode:(G3D_ShadeMode)mode;
- (G3D_ShadeMode)shadeMode;

- (void)setAlphaFunc:(G3D_AlphaFunc)func;
- (G3D_AlphaFunc)alphaFunc;

- (void)setAlphaClamp:(float)val;
- (float)alphaClamp;

//-----------------------------------------------------------------------------
// Activate and deactivate
//-----------------------------------------------------------------------------

- (BOOL)isFeatureEnabled:(G3D_RenderFeature)md;
- (void)enableFeature:(G3D_RenderFeature)md;
- (void)disableFeature:(G3D_RenderFeature)md;

//-----------------------------------------------------------------------------
// Accessor methods
//-----------------------------------------------------------------------------

- (void)setMaterial:(G3DMaterial *)mat;
- (G3DMaterial *)material;

- (void)setTextureHandle:(GLuint)handle;
- (GLuint)textureHandle;

- (NSString *)description;

//-----------------------------------------------------------------------------
// NSCopying
//-----------------------------------------------------------------------------

- (id)copyWithZone:(NSZone *)zone;
  // Never invoke copy directly! Instead you should copy the appr. shape object!

//-----------------------------------------------------------------------------
// NSCoding
//-----------------------------------------------------------------------------

- (void)encodeWithCoder:(NSCoder *)aCoder;
- (id)initWithCoder:(NSCoder *)aCoder;

//-----------------------------------------------------------------------------
// G3DSceneCoding
//-----------------------------------------------------------------------------

- (void)encodeWithSceneArchiver:(G3DArchiver *)archiver;
- (id)initWithSceneUnarchiver:(G3DUnarchiver *)unarchiver;

@end

extern NSString *G3DStateException;

#endif




