//-----------------------------------------------------------------------------
// Project            	RenderKit
// Class		G3DSurface
// Creator            	Philippe C.D. Robert
// Maintainer         	Philippe C.D. Robert
// Creation Date      	Fri Jun 16 13:50:36 CEST 2000
//
// Copyright (c) Philippe C.D. Robert
//
// The RenderKit is free software; you can redistribute it and/or modify it 
// under the terms of the GNU LGPL Version 2 as published by the Free 
// Software Foundation
//
// $Id: G3DSurface.h,v 1.6 2001/01/05 20:50:55 robert Exp $
//-----------------------------------------------------------------------------

/*
 * A surface is just a square plane, located at 0,0 with length 1. If scale
 * is not 1.0f the surface gets scaled in x and z direction.
 *
 * For the moment hte surface is just one quad. Later I will add strips
 * to make it better suited for lighting.
 *
 */

#ifndef __G3DSurface_h_INCLUDE
#define __G3DSurface_h_INCLUDE

#import <Foundation/Foundation.h>
#import <RenderKit/G3DGeometryBackend.h>
#import <RenderKit/G3DAttributeNode.h>

@class G3DShape;
@class G3DColour;

@interface G3DSurface : G3DAttributeNode<G3DGeometryBackend,NSCopying,NSCoding,G3DSceneCoding>
{
  float rgba[4];
  float _sc;
  G3DShape *shape;
  unsigned dl;

  @private
    
  BOOL _hasDL;
  float _tResX;
  float _tResY;
}

//-----------------------------------------------------------------------------
// Class methods
//-----------------------------------------------------------------------------

+ (G3DSurface *)surfaceWithScale:(float)sc;

//-----------------------------------------------------------------------------
// Init and free
//-----------------------------------------------------------------------------

- (id)initWithScale:(float)sc;

//-----------------------------------------------------------------------------
// Geometry backend
//-----------------------------------------------------------------------------

- (void)setTextureCoordinateX:(float)xval y:(float)yval;

- (void)drawGeometry;
- (void)precompile;

- (int)vertexCount;
- (void)getMin:(float *)min max:(float *)max;
- (void)setShape:(G3DShape *)aShape;

- (void)setColour:(G3DColour *)col;
- (G3DColour *)colour;

//-----------------------------------------------------------------------------
// NSCoding
//-----------------------------------------------------------------------------

- (void)encodeWithCoder:(NSCoder *)aCoder;
- (id)initWithCoder:(NSCoder *)aCoder;

//-----------------------------------------------------------------------------
// NSCopying
//-----------------------------------------------------------------------------

- (id)copyWithZone:(NSZone *)zone;

//-----------------------------------------------------------------------------
// G3DSceneCoding
//-----------------------------------------------------------------------------

- (void)encodeWithSceneArchiver:(G3DArchiver *)archiver;
- (id)initWithSceneUnarchiver:(G3DUnarchiver *)unarchiver;

@end

extern NSString *G3DSurfaceException;

#endif







