//-----------------------------------------------------------------------------
// Project            	RenderKit
// Class              	G3DTypes
// Creator            	Philippe C.D. Robert
// Maintainer         	Philippe C.D. Robert
// Creation Date      	Thu Sep  9 11:05:08 CEST 1999
//
// Copyright (c) Philippe C.D. Robert
//
// The RenderKit is free software; you can redistribute it and/or modify it 
// under the terms of the GNU LGPL Version 2 as published by the Free 
// Software Foundation
//
// $Id: G3DTypes.h,v 1.13 2001/07/15 12:39:24 probert Exp $
//
//-----------------------------------------------------------------------------


#ifndef __G3DTypes_h_INCLUDE
#define __G3DTypes_h_INCLUDE

#import <GeometryKit/G3DDefs.h>
#import <GeometryKit/G3DFunctions.h>

//-----------------------------------------------------------------------------
// Miscellaneous
//-----------------------------------------------------------------------------

typedef int G3DNodeType;

typedef struct {
  int off;
  int length;
} G3DRange;

typedef enum {
  G3DOutside = 0,
  G3DInside,
  G3DInbetween
} G3DLocation;

typedef enum {
  G3D_RealGeometry,
  G3D_BoundingVolume
} G3D_RenderMode;

typedef struct _G3DCoordinate {
  float position[4];
  float direction[4];
  float up[4];
} G3DCoordinate;

//-----------------------------------------------------------------------------
// Rendering
//-----------------------------------------------------------------------------

typedef enum {
  G3D_PointCloud,
  G3D_WireFrame,
  G3D_Shaded,
  G3D_Solid
} G3D_SurfaceType;

//-----------------------------------------------------------------------------
// Geometry
//-----------------------------------------------------------------------------

typedef struct _G3DBoundingSphere {
  double radius;
  double center[3];
} G3DBoundingSphere;

typedef struct _G3DBoundingBox {
  double front[3];
  double back[3];
} G3DBoundingBox;

//-----------------------------------------------------------------------------
// OpenGL stacks
//-----------------------------------------------------------------------------

typedef enum {
  G3D_ModelStack,
  G3D_TextureStack
} G3D_TransformStack;

//-----------------------------------------------------------------------------
// Camera
//-----------------------------------------------------------------------------

typedef struct _cameraPosition {
  float pos[3];
  float quat[4];
  float euler[16];
  float ieuler[16];
} G3D_CameraPosition;

typedef struct _cameraFlags {
  unsigned int degenerate:1;
  unsigned int basisChanged:1;
  unsigned int canRender:1;
  unsigned int usePreTM:1;
  unsigned int doFlush:1;
  unsigned int drawBackground:1;
  unsigned int isFullscreen:1;
  unsigned int doDepthTest:1;
} G3D_CameraFlags;

typedef enum {
  G3D_Fullscreen,
  G3D_Window
} G3D_DisplayMode;

#endif






