//-----------------------------------------------------------------------------
// Project            	RenderKit
// Class		G3DUnarchiver
// Creator            	Philippe C.D. Robert
// Maintainer         	Philippe C.D. Robert
// Creation Date      	Wed Nov 24 18:49:25 CET 1999
//
// Copyright (c) Philippe C.D. Robert
//
// The RenderKit is free software; you can redistribute it and/or modify it 
// under the terms of the GNU LGPL Version 2 as published by the Free 
// Software Foundation
//
// $Id: G3DUnarchiver.h,v 1.12 2001/07/07 15:35:47 probert Exp $
//-----------------------------------------------------------------------------

/*
 Description:

 */

#ifndef __G3DUnarchiver_h_INCLUDE
#define __G3DUnarchiver_h_INCLUDE

#import <Foundation/Foundation.h>
#import <RenderKit/G3DTypes.h>
#import <RenderKit/G3DSceneCoding.h>
#import <RenderKit/G3DFrustum.h>

@interface G3DUnarchiver : NSObject
{
  NSMutableDictionary *pList;
  id currentDecodedObjectRep;
  NSMutableDictionary *namesToObjects; // object name -> object

  int level;
  int version;

  NSZone *objectZone;
}

//-----------------------------------------------------------------------------
// Class methods
//-----------------------------------------------------------------------------

+ (void)initialize;

+ (id)unarchiveObjectTreeWithContentsOfFile:(NSString*)filename;

+ (id)unarchiveObjectNamed:(NSString*)name fromPropertyList:(id)pList;
+ (id)unarchiveObjectNamed:(NSString*)name fromFile:(NSString*)path;

+ (NSString*)decodedClassNameForArchiveClassName:(NSString*)archName;
+ (void)decodeClassName:(NSString*)archName asClassName:(NSString*)trueName;

//-----------------------------------------------------------------------------
// Init and free
//-----------------------------------------------------------------------------

- (id)init;
- (id)initForReadingWithPropertyList:(id)pList;
- (void)dealloc;

//-----------------------------------------------------------------------------
// Unarchiving
//-----------------------------------------------------------------------------

- (NSZone*)objectZone;
- (void)setObjectZone:(NSZone*)zone;
- (unsigned int)systemVersion;

- (NSString*)decodedClassNameForArchiveClassName:(NSString*)archName;
- (void)decodeClassName:(NSString*)archName asClassName:(NSString*)trueName;
- (unsigned int)versionForClassName:(NSString*)className;

//-----------------------------------------------------------------------------
// Decoding objects
//-----------------------------------------------------------------------------

- (id)decodeObjectNamed:(NSString*)name;
- (NSString*)decodeStringNamed:(NSString*)name;
- (NSArray*)decodeArrayNamed:(NSString*)name;
- (NSDictionary*)decodeDictionaryNamed:(NSString*)name;
- (NSData*)decodeDataNamed:(NSString*)name;
- (Class)decodeClassNamed:(NSString*)name;
- (SEL)decodeSelectorNamed:(NSString*)name;

//-----------------------------------------------------------------------------
// Decoding C Types
//-----------------------------------------------------------------------------

- (void)decodeBytesWithReturnedLength:(unsigned *)l named:(NSString *)name;

- (void)decodeArrayOfTypeDoubleAt:(double *)address count:(unsigned)cnt named:(NSString*)name;

- (void)decodeArrayOfTypeFloatAt:(float *)address count:(unsigned)cnt named:(NSString*)name;

- (void)decodeArrayOfTypeIntAt:(int *)address count:(unsigned)cnt named:(NSString*)name;

- (void)decodeChar:(char *)value named:(NSString*)name;
- (void)decodeUnsignedChar:(unsigned char *)value named:(NSString*)name;

- (void)decodeBOOL:(BOOL *)value named:(NSString*)name;

- (void)decodeShort:(short *)value named:(NSString*)name;
- (void)decodeUnsignedShort:(unsigned short *)value named:(NSString*)name;

- (void)decodeInt:(int *)value named:(NSString*)name;
- (void)decodeUnsignedInt:(unsigned *)value named:(NSString*)name;

- (void)decodeLong:(long *)value named:(NSString*)name;
- (void)decodeUnsignedLong:(unsigned long *)value named:(NSString*)name;

- (void)decodeFloat:(float *)value named:(NSString*)name;
- (void)decodeDouble:(double *)value named:(NSString*)name;

//-----------------------------------------------------------------------------
// Decoding other DTs
//-----------------------------------------------------------------------------

- (NSPoint)decodePointNamed:(NSString*)name;
- (NSSize)decodeSizeNamed:(NSString*)name;
- (NSRect)decodeRectNamed:(NSString*)name;

- (G3DFrustumValues)decodeFrustumNamed:(NSString*)name;
- (G3DCoordinate)decodeCoordNamed:(NSString*)name;
- (G3DRange)decodeRangeNamed:(NSString*)name;
- (G3DBoundingSphere)decodeBoundingSphereNamed:(NSString*)name;
- (G3DBoundingBox)decodeBoundingBoxNamed:(NSString*)name;

/*
- (G3D_CameraFlags)decodeCameraFlagsNamed:(NSString*)name;
*/

@end

#endif
