//-----------------------------------------------------------------------------
// Project            	RenderKit
// Class		G3DVtxArray
// Creator            	Philippe C.D. Robert
// Maintainer         	Philippe C.D. Robert
// Creation Date      	Thu Sep  9 11:05:08 CEST 1999
//
// Copyright (c) Philippe C.D. Robert
//
// The RenderKit is free software; you can redistribute it and/or modify it 
// under the terms of the GNU LGPL Version 2 as published by the Free 
// Software Foundation
//
// $Id: G3DVtxArray.h,v 1.14 2001/01/05 20:50:56 robert Exp $
//-----------------------------------------------------------------------------

/*
 * This class is used by G3DShape to store all the geometry. It is a wrapper
 * for vertex arrays.
 *
 */

#ifndef __G3DVtxArray_h_INCLUDE
#define __G3DVtxArray_h_INCLUDE

#import <Foundation/Foundation.h>
#import <RenderKit/G3DTypes.h>
#import <RenderKit/G3DAttributeNode.h>
#import <RenderKit/G3DGeometryBackend.h>

#define SIZE_VERTEX 4
#define SIZE_COLOUR 4
#define SIZE_NORMAL 3
#define SIZE_TCOORD 2

@class G3DShape;

@interface G3DVtxArray : G3DAttributeNode <NSCoding,NSCopying,G3DGeometryBackend,G3DSceneCoding>
{
  int vertices;
  int geometryMode;     // G3DGeometryMode

  float *vertexArray; 	// The vertex arrays
  float *colourArray;
  float *normalArray;
  float *texCoordArray;

  G3DShape *shape;	// Gets informed to update its bVolume, if needed
  BOOL isLocked;

  BOOL _hasNormals;
  BOOL _hasColours;
  BOOL _hasTexCoords;
}

//-----------------------------------------------------------------------------
// Init
//-----------------------------------------------------------------------------

- (id)initWithMode:(G3DGeometryMode)mode count:(int)cnt;

//-----------------------------------------------------------------------------
// Drawing
//-----------------------------------------------------------------------------

- (void)drawGeometry;
  // Is invoked by its G3DShape to render the arrays

//-----------------------------------------------------------------------------
// Accessor Methods
//-----------------------------------------------------------------------------

- (void)precompile;
  // Makes the array locked, if possible

- (int)vertexCount;

- (float *)colourArray;
- (float *)vertexArray;
- (float *)normalArray;
- (float *)texCoordArray;

- (G3DGeometryMode)geometryMode;

- (void)getMin:(float *)min max:(float *)max;
  // Fills the 2 float arrays of size ver.length with the 'virtual' min and the max of the space that is occupied by this vertex array. This canbeused to calculate an AABB.

- (void)setShape:(G3DShape *)aShape;
  // Never invoke this directly! This is private!

- (void)setUseVertexNormals:(BOOL)yn;

//-----------------------------------------------------------------------------
// Geometry
//-----------------------------------------------------------------------------

- (void)setCoordinate:(float *)coord atIndex:(int)index;
- (float *)coordinateAtIndex:(int)index;

- (void)setCoordinates:(float *)coords atIndex:(int)index count:(int)cnt;
- (void)setCoordinates:(float *)coords range:(NSRange)rng;

- (void)setNormal:(float *)coord atIndex:(int)index;
- (float *)normalAtIndex:(int)index;

- (void)setNormals:(float *)coords atIndex:(int)index count:(int)cnt;
- (void)setNormals:(float *)coords range:(NSRange)rng;

//-----------------------------------------------------------------------------
// Colours
//-----------------------------------------------------------------------------

- (void)setColour:(float *)col atIndex:(int)index;
- (float *)colourAtIndex:(int)index;

- (void)setColours:(float *)cols atIndex:(int)index count:(int)cnt;
- (void)setColours:(float *)cols range:(NSRange)rng;

//-----------------------------------------------------------------------------
// Texture coordinates
//-----------------------------------------------------------------------------

- (void)setTexCoord:(float *)coord atIndex:(int)index;
- (float *)texCoordAtIndex:(int)index;

- (void)setTexCoords:(float *)coords atIndex:(int)index count:(int)cnt;
- (void)setTexCoords:(float *)coords range:(NSRange)rng;

//-----------------------------------------------------------------------------
// NSCoding
//-----------------------------------------------------------------------------

- (void)encodeWithCoder:(NSCoder *)aCoder;
- (id)initWithCoder:(NSCoder *)aCoder;

//-----------------------------------------------------------------------------
// NSCopying
//-----------------------------------------------------------------------------

- (id)copyWithZone:(NSZone *)zone;

//-----------------------------------------------------------------------------
// G3DSceneCoding
//-----------------------------------------------------------------------------

- (void)encodeWithSceneArchiver:(G3DArchiver *)archiver;
- (id)initWithSceneUnarchiver:(G3DUnarchiver *)unarchiver;

@end

extern NSString *G3DVtxArrayException;
extern NSString *G3DVtxArrayFormatException;
extern NSString *G3DVtxArrayLockingException;

#endif




