//-----------------------------------------------------------------------------
// Project            	RenderKit
// Class		G3DWorld
// Creator            	Philippe C.D. Robert
// Maintainer         	Philippe C.D. Robert
// Creation Date      	Thu Sep  9 11:05:08 CEST 1999
//
// Copyright (c) Philippe C.D. Robert
//
// The RenderKit is free software; you can redistribute it and/or modify it 
// under the terms of the GNU LGPL Version 2 as published by the Free 
// Software Foundation
//
// $Id: G3DWorld.h,v 1.21 2001/07/07 15:35:47 probert Exp $
//-----------------------------------------------------------------------------

#ifndef __G3DWorld_h_INCLUDE
#define __G3DWorld_h_INCLUDE

#import <Foundation/Foundation.h>
#import <RenderKit/G3DGraphNode.h>

@class G3DNodeList;
@class G3DGroup;

@interface G3DWorld : G3DGraphNode <NSCoding,NSCopying,G3DSceneCoding>
{
  G3DGroup *staticObjects;
  G3DGroup *dynamicObjects;	// Leaves should be cached!
  G3DGroup *overlays;

  NSMutableArray *_dLights;

  @private

  int _s_objs;
  int _d_objs;
  int _d_lights;
  int _overlays;
  
  BOOL _useFog;

  //G3DGraphUpdater *updater;
}

//-----------------------------------------------------------------------------
// Class methods
//-----------------------------------------------------------------------------

+ (void)setDefaultWorldClass:(Class)rootClass;
+ (Class)defaultWorldClass;

+ (void)setUseMultipleThreads:(BOOL)yn;

+ (G3DWorld *)world;

//-----------------------------------------------------------------------------
// Init methods
//-----------------------------------------------------------------------------

- (id)init;

//-----------------------------------------------------------------------------
// Rendering
//-----------------------------------------------------------------------------

- (void)preprocess;

//-----------------------------------------------------------------------------
// The parent and the children
//-----------------------------------------------------------------------------

- (G3DGroup *)staticObjects;
- (void)addStaticObject:(G3DGraphNode *)aChild;
- (void)removeStaticObject:(G3DGraphNode *)aChild;

- (G3DGroup *)dynamicObjects;
- (void)addDynamicObject:(G3DGraphNode *)aChild;
- (void)removeDynamicObject:(G3DGraphNode *)aChild;

- (G3DGroup *)overlays;
- (void)addOverlay:(G3DGraphNode *)aChild;
- (void)removeOverlay:(G3DGraphNode *)aChild;

//-----------------------------------------------------------------------------
// NSCoding
//-----------------------------------------------------------------------------

- (void)encodeWithCoder:(NSCoder *)aCoder;
- (id)initWithCoder:(NSCoder *)aCoder;

//-----------------------------------------------------------------------------
// NSCopying
//-----------------------------------------------------------------------------

- (id)copyWithZone:(NSZone *)zone;

//-----------------------------------------------------------------------------
// G3DSceneCoding
//-----------------------------------------------------------------------------

- (void)encodeWithSceneArchiver:(G3DArchiver *)archiver;
- (id)initWithSceneUnarchiver:(G3DUnarchiver *)unarchiver;

@end

extern NSString *G3DWorldParentException;

#endif



